/*!
 * \file fsUtil.hpp
 * \brief This file is utilities to access file system.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */
#ifndef _FS_UTIL_H
#define _FS_UTIL_H

#include <sys/types.h>

/*!
 * \brief Copy data as avoid overwriting.
 * \param sourceFile [in] Path of source file.
 * \param destPath   [in] Path of directory put duplicated file.
 * \param destName   [in] Name of duplicated file.<br>
 *                        Don't rename, if value is null.
 * \return Prosess is succeed or failure.
 */
bool copyFile(char const* sourceFile,
                       char const* destPath, char const* destName = NULL);

/*!
 * \brief Create filename in expected path.
 * \param basePath [in] Path of directory.
 * \param filename [in] filename.
 * \return Create string.<br>
 *         Process is failure, if return is null.<br>
 *         Need call "free", if return is not null.
 */
char *createFilename(char const* basePath, char const* filename);

/*!
 * \brief Check link is regular file.
 * \param linkPath [in]  Path of link file.
 * \param filePath [out] Path of file is linked by link file.
 * \param nameSize [in]  Max size of paramter "filePath".
 * \return Value is true, if link is linked regular file.<br>
 *         Value is false, if link is linked directory or other.
 */
bool isCopyableLink(char const* linkPath, char *filePath, size_t nameSize);

/*!
 * \brief Create temporary directory in current directory.
 * \param basePath   [out] Path of temporary directory.<br>
 *                         Process is failure, if return is null.<br>
 *                         Need call "free", if return is not null.
 * \param wishesName [in]  Name of one's wishes directory name.
 */
void createTempDir(char **basePath, char const* wishesName);

/*!
 * \brief Remove temporary directory.
 * \param basePath [in] Path of temporary directory.
 */
void removeTempDir(char const* basePath);

/*!
 * \brief Create unique path.
 * \param path        [in] Path.
 * \param isDirectory [in] Path is directory.
 * \return Unique path.<br>Don't forget deallocate.
 */
char *createUniquePath(char *path, bool isDirectory);

/*!
 * \brief Get parent directory path.
 * \param path [in] Path which file or directory.
 * \return Parent directory path.<br>Don't forget deallocate.
 */
char *getParentDirectoryPath(char const* path);

/*!
 * \brief Get accessible of directory.
 * \param path      [in] Directory path.
 * \param needRead  [in] Accessible flag about file read.
 * \param needWrite [in] Accessible flag about file write.
 * \return This value is true, if directory path is accessible.<br>
 *         This value is false, if process is failure or path is inaccessible.
 */
bool isAccessibleDirectory(char const* path, bool needRead, bool needWrite);

/*!
 * \brief Mutex of working directory.
 */
extern pthread_mutex_t directoryMutex;

#endif // _FS_UTIL_H
