/*!
 * \file gcWatcher.hpp
 * \brief This file is used to take snapshot when finish garbage collection.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */

#ifndef GCWATCHER_HPP
#define GCWATCHER_HPP

#include <jni.h>
#include <jvmti.h>

#include "agentThread.hpp"
#include "util.hpp"
#include "jvmInfo.hpp"

/*!
 * \brief This type is callback to notify finished gc by gcWatcher.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 * \param cause [in] Cause of invoke function.<br>Maybe "GC".
 */
typedef void (*TPostGCFunc)(jvmtiEnv *jvmti, JNIEnv *env, TInvokeCause cause);

/*!
 * \brief This class is used to take snapshot when finished GC.
 */
class TGCWatcher : public TAgentThread {
  public:
    /*!
     * \brief TGCWatcher constructor.
     * \param postGCFunc [in] Callback is called after GC.
     * \param info       [in] JVM running performance information.
     */
    TGCWatcher(TPostGCFunc postGCFunc, TJvmInfo *info);
    /*!
     * \brief TGCWatcher destructor.
     */
    virtual ~TGCWatcher(void);
    
    /*!
     * \brief What's gc kind.
     * \return is full-GC? (true/false)
     */
    inline bool needToStartGCTrigger(void){
      bool isFullGC;
      /* Check full-GC count. */
      if (this->pJvmInfo->getFGCCount() > this->_FGC) {
        
        /* Refresh full GC count. */
        this->_FGC = this->pJvmInfo->getFGCCount();
        /* GC kind is full-GC. */
        isFullGC = true;
      } else {
        
        /* GC kind is young-GC. */
        isFullGC = false;
      }
      
      /* Set GC kind flag. */
      this->pJvmInfo->setFullGCFlag(isFullGC);
      return isFullGC;
    };
    
    /*!
     * \brief Make and begin Jthread.
     * \param jvmti [in] JVMTI environment object.
     * \param env   [in] JNI environment object.
     */
    void start(jvmtiEnv *jvmti, JNIEnv *env);
    
  protected:
    /*!
     * \brief JThread entry point.
     * \param jvmti [in] JVMTI environment object.
     * \param jni   [in] JNI environment object.
     * \param data  [in] Pointer of user data.
     */
    static void JNICALL entryPoint(jvmtiEnv *jvmti, JNIEnv *jni, void *data);
    
  private:
    /*!
     * \brief Treated full-GC count.
     */
    jlong _FGC;
    /*!
     * \brief Callback for notify finished gc by gcWatcher.
     */
    TPostGCFunc _postGCFunc;
    
    /*!
     * \brief JVM running performance information.
     */
    TJvmInfo *pJvmInfo;
};

#endif
