/*!
 * \file jvmInfo.cpp
 * \brief This file is used to get JVM performance information.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */

#include <jni.h>
#include <jvmti.h>

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <dlfcn.h>

#ifndef USE_VMSTRUCTS
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#endif

#include "jvmInfo.hpp"
#include "util.hpp"
#include "agentThread.hpp"

#ifdef USE_VMSTRUCTS
#include "vmStructScanner.hpp"
#endif

/*!
 * \brief TJvmInfo constructor.
 */
TJvmInfo::TJvmInfo(void) : TAgentThread("HeapStats JVM PerformInfo") {
  
  /* Initialization. */
  _nowFGC      = NULL;
  _nowYGC      = NULL;
  _edenSize    = NULL;
  _sur0Size    = NULL;
  _sur1Size    = NULL;
  _oldSize     = NULL;
  _FGCTime     = NULL;
  _YGCTime     = NULL;
  _freqTime    = NULL;
  _gcCause     = NULL;
  _lastGCCause = NULL;
  perfAddr     = 0;
  elapFGCTime  = 0;
  elapYGCTime  = 0;
  fullgcFlag   = false;
  loadLogFlag      = false;
  loadDelayLogFlag = false;
  
  /* Logging information. */
  _syncPark      = NULL;
  _threadLive    = NULL;
  _safePointTime = NULL;
  _safePoints    = NULL;
  _vmVersion     = NULL;
  _vmName        = NULL;
  _classPath     = NULL;
  _endorsedPath  = NULL;
  _javaVersion   = NULL;
  _javaHome      = NULL;
  _bootClassPath = NULL;
  _vmArgs        = NULL;
  _vmFlags       = NULL;
  _javaCommand   = NULL;
  _tickTime      = NULL;
  
  /* Allocate GC cause container. */
#ifdef AVX
  if (likely(usableAVX)) {
    if (unlikely(posix_memalign((void **)&gcCause, 32, /* for vmovdqa. */
      MAXSIZE_GC_CAUSE) != 0)) {
      throw "Couldn't allocate gc-cause memory!";
    }
  } else
#endif // AVX
#ifdef SSE2
  if (likely(usableSSE2)) {
    if (unlikely(posix_memalign((void **)&gcCause, 16, /* for movdqa. */
      MAXSIZE_GC_CAUSE) != 0)) {
      throw "Couldn't allocate gc-cause memory!";
    }
  } else {
#endif // SSE2
    gcCause = (char *)calloc(1, MAXSIZE_GC_CAUSE);
#ifdef SSE2
  }
#endif // SSE2
  
  /* If failure allocate GC cause container. */
  if (unlikely(gcCause == NULL)) {
    throw "Couldn't allocate gc-cause memory!";
  }
  
  /* Initialize. */
  memcpy(this->gcCause, UNKNOWN_GC_CAUSE, 16);
}

/*!
 * \brief TJvmInfo destructor.
 */
TJvmInfo::~TJvmInfo() {
  /* Free allcated GC cause container. */
  free(gcCause);
}

/*!
 * \brief Start parallel work by JThread.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 */
void TJvmInfo::start(jvmtiEnv *jvmti, JNIEnv *env) {
  /* Get GC information addres. */
  #ifdef USE_VMSTRUCTS
    this->detectInfoAddress();
  #else
    this->detectInfoAddress(env);
  #endif
  
  /* Get memory function address. */
  this->maxMemFunc   = (TGetMemoryFunc)dlsym(RTLD_DEFAULT, "JVM_MaxMemory");
  this->totalMemFunc = (TGetMemoryFunc)dlsym(RTLD_DEFAULT, "JVM_TotalMemory");
  
  /* Check memory function address. */
  if (this->maxMemFunc == NULL) {
    PRINT_WARN_MSG("Couldn't get memory function. function:JVM_MaxMemory");
  }
  
  if (this->totalMemFunc == NULL) {
    PRINT_WARN_MSG("Couldn't get memory function. function:JVM_TotalMemory");
  }
  
  /* Start JThread. */
  TAgentThread::start(jvmti, env, TJvmInfo::entryPoint,
                                            this, JVMTI_THREAD_MIN_PRIORITY);
}

/*!
 * \brief Parallel work function by JThread.
 * \param jvmti [in] JVMTI environment object.
 * \param jni   [in] JNI environment object.
 * \param data  [in] Pointer of TJvmInfo.
 */
void JNICALL TJvmInfo::entryPoint(jvmtiEnv *jvmti, JNIEnv *jni, void *data) {
  /* Get self. */
  TJvmInfo *controller = (TJvmInfo *)data;
  /* Change running state. */
  controller->_isRunning = true;
  
  /* Loop for agent run. */
  while (!controller->_terminateRequest) {
    bool needProcess = false;
    
    ENTER_PTHREAD_SECTION(&controller->mutex) {
      
      /* Wait for notification or termination. */
      pthread_cond_wait(&controller->mutexCond, &controller->mutex);
      
      /* If get notification */
      if (likely(controller->_numRequests > 0)) {
        controller->_numRequests--;
        needProcess = true;
      }
      
    } EXIT_PTHREAD_SECTION(&controller->mutex)
    
    /* If waiting is finished by notification. */
    if (needProcess) {
      /* Get delay info */
      if (unlikely(controller->_freqTime == NULL)) {
        controller->detectDelayInfoAddress();
      }
      /* Load string of GC cause. */
      controller->loadGCCause();
    }
  }
  
  /* Change running state. */
  controller->_isRunning = false;
}

#ifdef USE_VMSTRUCTS

/*!
 * \brief Get JVM performance data address.<br>
 *        Use external VM structure.
 * \return JVM performance data address.
 */
ptrdiff_t TJvmInfo::getPerfMemoryAddress(void) {
  /* Symbol "gHotSpotVMStructs" is included in "libjvm.so". */
  extern VMStructEntry *gHotSpotVMStructs;
  /* If don't exist gHotSpotVMStructs. */
  if (gHotSpotVMStructs == NULL) {
    return 0;
  }
  
  /* Search perfomance data in gHotSpotVMStructs. */
  int Cnt = 0;
  ptrdiff_t addr = 0;
  while (gHotSpotVMStructs[Cnt].typeName != NULL) {
    /* Check each entry. */
    if ((strncmp(gHotSpotVMStructs[Cnt].typeName, "PerfMemory", 10) == 0)
            && (strncmp(gHotSpotVMStructs[Cnt].fieldName, "_start", 6) == 0)) {
      
      /* Found. */
      addr = (ptrdiff_t)*(char **)gHotSpotVMStructs[Cnt].address;
      break;
    }
    
    /* Go next entry. */
    Cnt++;
  }
  
  return addr;
}

#else

/*!
 * \brief Search JVM performance data address in memory.
 * \param path [in] Path of hsperfdata.
 * \return JVM performance data address.
 */
ptrdiff_t TJvmInfo::findPerfMemoryAddress(char *path) {
  /* Search entries in performance file. */
  ptrdiff_t addr = 0;
  ptrdiff_t start_addr;
  char mapPath[101];
  char file[PATH_MAX + 1];
  char *line = NULL;
  char lineFmt[101];
  size_t len = 0;
  
  /* Decode link in path. */
  char target[PATH_MAX + 1] = {0};
  if (unlikely(realpath(path, target) == NULL)) {
    return 0;
  }
  
  /* Open process map. */
  snprintf(mapPath, 100, "/proc/%d/maps", getpid());
  FILE *maps = fopen(mapPath, "r");
  
  /* If failure open file. */
  if (unlikely(maps == NULL)) {
    return 0;
  }
  
  snprintf(lineFmt, 100, "%%tx-%%*x %%*s %%*x %%*x:%%*x %%*u %%%ds", PATH_MAX);
  /* Repeats until EOF. */
  while (getline(&line, &len, maps) != -1) {
    /*
     * From Linux Kernel source.
     * fs/proc/task_mmu.c: static void show_map_vma()
     */
    if (sscanf(line, lineFmt, &start_addr, file) == 2) {
      
      /* If found load address. */
      if (strcmp(file, target) == 0) {
        addr = start_addr;
        break;
      }
    }
  }
  
  /* Clean up after. */
  if (line != NULL) {
    free(line);
  }
  
  /* Close map. */
  fclose(maps);
  return addr;
}

/*!
 * \brief Get JVM performance data address.<br>
 *        Use performance data file.
 * \param env [in] JNI environment object.
 * \return JVM performance data address.
 */
ptrdiff_t TJvmInfo::getPerfMemoryAddress(JNIEnv *env) {
  
  /* Get perfomance file path. */
  
  char perfPath[PATH_MAX];
  char tempPath[PATH_MAX];
  char *tmpdir    = NULL;
  char *username  = NULL;
  char *separator = NULL;
  ptrdiff_t addr  = 0;
  
  /* GetSystemProperty() on JVMTI gets value from Arguments::_system_properties
   * ONLY.
   * Arguments::_system_properties differents from
   * java.lang.System.getProperties().
   * So, this function uses java.lang.System.getProperty(String).
   */
  tmpdir    = GetSystemProperty(env, "java.io.tmpdir");
  username  = GetSystemProperty(env, "user.name");
  separator = GetSystemProperty(env, "file.separator");
  
  /* Value check. */
  if ((tmpdir == NULL) || (username == NULL) || (separator == NULL)) {
    /* Free allocated memories. */
    if (tmpdir != NULL) {
      free(tmpdir);
    }
    if (username != NULL) {
      free(username);
    }
    if (separator != NULL) {
      free(separator);
    }
    return 0;
  }
  
  /* Check and fix path. e.g "/usr/local/" */
  if (strlen(tmpdir) > 0 && tmpdir[strlen(tmpdir) - 1] == '/') {
    tmpdir[strlen(tmpdir) - 1] = '\0';
  }
  
  /* Make performance file path. */
  sprintf(perfPath, "%s%shsperfdata_%s%s%d", tmpdir, separator,
                                            username, separator, getpid());
  sprintf(tempPath, "%stmp%shsperfdata_%s%s%d", separator, separator,
                                            username, separator, getpid());
  
  /* Clean up after get performance file path. */
  free(tmpdir);
  free(username);
  free(separator);
  
  /* Search entries in performance file. */
  
  addr = findPerfMemoryAddress(perfPath);
  if (addr == 0) {
    addr = findPerfMemoryAddress(tempPath);
  }
  
  return addr;
}

#endif

#ifdef USE_VMSTRUCTS

/*!
 * \brief Detect GC-informaion address.<br>
 *        Use external VM structure.
 * \sa    "jdk/src/share/classes/sun/tools/jstat/resources/jstat_options"
 */
void TJvmInfo::detectInfoAddress(void) {
  
  /* Get performance address. */
  this->perfAddr = this->getPerfMemoryAddress();
#else

/*!
 * \brief Detect GC-informaion address.<br>
 *        Use performance data file.
 * \param env [in] JNI environment object.
 * \sa    "jdk/src/share/classes/sun/tools/jstat/resources/jstat_options"
 */
void TJvmInfo::detectInfoAddress(JNIEnv *env) {
  
  /* Get performance address. */
  this->perfAddr = this->getPerfMemoryAddress(env);
#endif
  
  /* Check result. */
  if (this->perfAddr == 0) {
  #ifdef USE_VMSTRUCTS
    PRINT_WARN_MSG("Necessary information isn't found in libjvm.so.");
  #else
    PRINT_WARN_MSG("Necessary information isn't found in performance file.");
  #endif
    return;
  }
  
  /* Copy performance data's header. */
  PerfDataPrologue prologue;
  memcpy(&prologue, (void *)perfAddr, sizeof(PerfDataPrologue));
  
  /* Check performance data's header. */
#ifdef WORDS_BIGENDIAN
  if (prologue.magic != (jint)0xcafec0c0) {
#else
  if (prologue.magic != (jint)0xc0c0feca) {
#endif
    PRINT_WARN_MSG("Performance data's magic is broken.");
    this->perfAddr = 0;
    return;
  }
  
  ptrdiff_t ofs;
  int name_len;
  char *name;
  PerfDataEntry entry;
  void *readPtr;
  
  /* Seek entries head. */
  readPtr = (void *)(perfAddr + prologue.entry_offset);
  
  /* Search target entries. */
  for (int Cnt = 0; Cnt < prologue.num_entries; Cnt++) {
    /* Copy entry data. */
    ofs = (ptrdiff_t)readPtr;
    memcpy(&entry, readPtr, sizeof(PerfDataEntry));
    
    /* Get entry name. */
    name_len = entry.data_offset - entry.name_offset;
    name = (char *)calloc(name_len + 1, sizeof(char));
    if (unlikely(name == NULL)) {
      PRINT_WARN_MSG("Couldn't allocate entry memory!");
      break;
    }
    
    readPtr = (void *)(ofs + entry.name_offset);
    memcpy(name, readPtr, name_len);
    
    /* Entries name check. */
    if ((strcmp("sun.gc.collector.0.invocations", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Young GC count entry. */
      this->_nowYGC = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.collector.1.invocations", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Full GC count entry. */
      this->_nowFGC = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.generation.0.space.0.used", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Eden using size entry. */
      this->_edenSize = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.generation.0.space.1.used", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Survivor0 using size entry. */
      this->_sur0Size = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.generation.0.space.2.used", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Survivor1 using size entry. */
      this->_sur1Size = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.generation.1.space.0.used", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Old using size entry. */
      this->_oldSize = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.collector.0.time", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Young GC work time entry. */
      this->_YGCTime = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.collector.1.time", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Full GC work time entry. */
      this->_FGCTime = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.cause", name) == 0) && (entry.data_type == 'B')) {
      /* GC cause entry. */
      this->_gcCause = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.gc.lastCause", name) == 0)
                                           && (entry.data_type == 'B')) {
      /* Last GC cause entry. */
      this->_lastGCCause = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.rt._sync_Parks", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Monitor conflict count entry. */
      this->_syncPark = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.threads.live", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Living thread count entry. */
      this->_threadLive = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.rt.safepointTime", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Safe point time entry. */
      this->_safePointTime = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.rt.safepoints", name) == 0)
                                           && (entry.data_type == 'J')) {
      /* Safe point count entry. */
      this->_safePoints = (jlong *)(ofs + entry.data_offset);
    }
    
    /* Clean up and move next entry. */
    free(name);
    readPtr = (void *)(ofs + entry.entry_length);
  }
  
  /* Verify necessary information. */
  
  /* If failure get young-GC count entry. */
  if (_nowYGC == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                   " Entry:\"sun.gc.collector.0.invocations\"");
  }
  
  /* If failure get full-GC count entry. */
  if (_nowFGC == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                   " Entry:\"sun.gc.collector.1.invocations\"");
  }
  
  /* If failure get Eden using area size entry. */
  if (_edenSize == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                 " Entry:\"sun.gc.generation.0.space.0.used\"");
  }
  
  /* If failure get Survivor0 area using size entry. */
  if (_sur0Size == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                 " Entry:\"sun.gc.generation.0.space.1.used\"");
  }
  
  /* If failure get Survivor1 area using size entry. */
  if (_sur1Size == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                 " Entry:\"sun.gc.generation.0.space.2.used\"");
  }
  
  /* If failure get old area using size entry. */
  if (_oldSize == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                 " Entry:\"sun.gc.generation.1.space.0.used\"");
  }
  
  /* If failure get young-GC work time entry. */
  if (_YGCTime == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                          " Entry:\"sun.gc.collector.0.time\"");
  }
  
  /* If failure get young-GC work time entry. */
  if (_FGCTime == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                          " Entry:\"sun.gc.collector.1.time\"");
  }
  
  /* If failure get GC cause entry. */
  if (_gcCause == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                     " Entry:\"sun.gc.cause\"");
  }
  
  /* If failure get last GC cause entry. */
  if (_lastGCCause == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                 " Entry:\"sun.gc.lastCause\"");
  }
  
  /* If failure get monitor conflict count entry. */
  if (_syncPark == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                               " Entry:\"sun.rt._sync_Parks\"");
  }
  
  /* If failure get living thread count entry. */
  if (_threadLive == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                " Entry:\"java.threads.live\"");
  }
  
  /* If failure get safe point time entry. */
  if (_safePointTime == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                             " Entry:\"sun.rt.safepointTime\"");
  }
  
  /* If failure get safe point count entry. */
  if (_safePoints == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                " Entry:\"sun.rt.safepoints\"");
  }
  
  /* Refresh log data load flag. */
  loadLogFlag = (_syncPark != NULL) && (_threadLive != NULL)
                         && (_safePointTime   != NULL) && (_safePoints != NULL);
}

/*!
 * \brief Detect delayed GC-informaion address.
 */
void TJvmInfo::detectDelayInfoAddress(void) {
  
  /* Check performance data's address. */
  if (this->perfAddr == 0) {
    return;
  }
  
  /* Copy performance data's header. */
  PerfDataPrologue prologue;
  memcpy(&prologue, (void *)perfAddr, sizeof(PerfDataPrologue));
  
  /* Search entry. */
  
  ptrdiff_t ofs;
  int name_len;
  char *name;
  PerfDataEntry entry;
  void *readPtr;
  
  /* Seek entries head. */
  readPtr = (void *)(perfAddr + prologue.entry_offset);
  
  /* Search target entries. */
  for (int Cnt = 0; Cnt < prologue.num_entries; Cnt++) {
    /* Copy entry data. */
    ofs = (ptrdiff_t)readPtr;
    memcpy(&entry, readPtr, sizeof(PerfDataEntry));
    
    /* Get entry name. */
    name_len = entry.data_offset - entry.name_offset;
    name = (char *)calloc(name_len, 1);
    if (unlikely(name == NULL)) {
      PRINT_WARN_MSG("Couldn't allocate entry memory!");
      break;
    }
    
    readPtr = (void *)(ofs + entry.name_offset);
    memcpy(name, readPtr, name_len);
    
    /* Entries name check. */
    if ((strcmp("sun.os.hrt.frequency", name) == 0)
                                      && (entry.data_type == 'J')) {
      /* Time tick frequency entry. */
      this->_freqTime = (jlong *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.property.java.vm.version", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* JVM version string entry. */
      this->_vmVersion = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.property.java.vm.name", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* JVM name entry. */
      this->_vmName = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.property.java.class.path", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* JVM class path entry. */
      this->_classPath = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.property.java.endorsed.dirs", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* Java endorsed directory entry. */
      this->_endorsedPath = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.property.java.version", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* Java version entry. */
      this->_javaVersion = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.property.java.home", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* JAVA_HOME entry. */
      this->_javaHome = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.property.sun.boot.class.path", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* Boot classpath entry. */
      this->_bootClassPath = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.rt.vmArgs", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* JVM arguments string entry. */
      this->_vmArgs = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("java.rt.vmFlags", name) == 0)
      && (entry.data_type == 'B')) {
      /* JVM flags entry. */
      this->_vmFlags = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.rt.javaCommand", name) == 0)
                                      && (entry.data_type == 'B')) {
      /* Java application arguments entry. */
      this->_javaCommand = (char *)(ofs + entry.data_offset);
    }
    else if ((strcmp("sun.os.hrt.ticks", name) == 0)
      && (entry.data_type == 'J')) {
      /* JVM running time entry. */
      this->_tickTime = (jlong *)(ofs + entry.data_offset);
    }
    
    /* Clean up and move next entry. */
    free(name);
    readPtr = (void *)(ofs + entry.entry_length);
  }
  
  /* Verify necessary delayed information. */
  
  if (_freqTime == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                             " Entry:\"sun.os.hrt.frequency\"");
  }
  
  /* If failure get JVM version string entry. */
  if (_vmVersion == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                    " Entry:\"java.property.java.vm.version\"");
  }
  
  /* If failure get JVM name entry. */
  if (_vmName == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                       " Entry:\"java.property.java.vm.name\"");
  }
  
  /* If failure get JVM class path entry. */
  if (_classPath == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                    " Entry:\"java.property.java.class.path\"");
  }
  
  /* If failure get java endorsed directory entry. */
  if (_endorsedPath == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                 " Entry:\"java.property.java.endorsed.dirs\"");
  }
  
  /* If failure get java version entry. */
  if (_javaVersion == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                       " Entry:\"java.property.java.version\"");
  }
  
  /* If failure get JAVA_HOME entry. */
  if (_javaHome == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                          " Entry:\"java.property.java.home\"");
  }
  
  /* If failure get boot classpath entry. */
  if (_bootClassPath == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                 " Entry:\"sun.property.sun.boot.class.path\"");
  }
  
  /* If failure get JVM arguments string entry. */
  if (_vmArgs == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                   " Entry:\"java.rt.vmArgs\"");
  }
  
  /* If failure get JVM flags entry. */
  if (_vmFlags == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                  " Entry:\"java.rt.vmFlags\"");
  }
  
  /* If failure get java application arguments entry. */
  if (_javaCommand == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                               " Entry:\"sun.rt.javaCommand\"");
  }
  
  /* If failure get JVM running time entry. */
  if (_tickTime == NULL) {
    PRINT_WARN_MSG("Necessary information isn't found in performance data."
                                                 " Entry:\"sun.os.hrt.ticks\"");
  }
  
  /* Refresh delay log data load flag. */
  loadDelayLogFlag =     (_freqTime     != NULL) && (_vmVersion     != NULL)
                      && (_vmName       != NULL) && (_classPath     != NULL)
                      && (_endorsedPath != NULL) && (_javaVersion   != NULL)
                      && (_javaHome     != NULL) && (_bootClassPath != NULL)
                      && (_vmArgs       != NULL) && (_vmFlags       != NULL)
                      && (_javaCommand  != NULL) && (_tickTime      != NULL);
}
