/*!
 * \file logmain.cpp
 * \brief This file is used common logging process.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */

#include <jni.h>
#include <jvmti.h>
#include <sys/time.h>
#include <pthread.h>
#include <signal.h>

#include "logMain.hpp"
#include "libmain.hpp"
#include "util.hpp"
#include "timer.hpp"
#include "signalManager.hpp"
#include "elapsedTimer.hpp"
#include "logManager.hpp"
#include "deadlockFinder.hpp"

/*!
 * \brief Manager of log information.
 */
TLogManager *logManager = NULL;
/*!
 * \brief Timer for interval collect log.
 */
TTimer *logTimer = NULL;
/*!
 * \brief Signal manager to collect normal log by signal.
 */
TSignalManager *logSignalMngr = NULL;
/*!
 * \brief Signal manager to collect all log by signal.
 */
TSignalManager *logAllSignalMngr = NULL;
/*!
 * \brief Mutex to avoid conflict in OnResourceExhausted.<br>
 * <br>
 * This mutex used in below process.<br>
 *   - OnResourceExhausted @ logMain.cpp<br>
 *     To read and write to flag mean already collect log
 *     on resource exhasuted.<br>
 */
pthread_mutex_t errMutex = PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP;

/*!
 * \brief Flag of collect normal log for signal.
 */
volatile sig_atomic_t flagLogSignal;
/*!
 * \brief Flag of collect all log for signal.
 */
volatile sig_atomic_t flagAllLogSignal;

/*!
 * \brief Deadlock finder for logging.
 */
TDeadlockFinder *lockFinder = NULL;

/*!
 * \brief Take log information.
 * \param jvmti   [in] JVMTI environment object.
 * \param env     [in] JNI environment object.
 * \param cause   [in] Cause of taking a snapshot.<br>
 *                     e.g. ResourceExhausted, Signal or Interval.
 * \param nowTime [in] Mili-second elapsed from 1970/1/1 0:00:00.<br>
 *                     This value express time of call log function.
 * \return Value is true, if process is succeed.
 */
inline bool TakeLogInfo(jvmtiEnv *jvmti, JNIEnv* env,
                                    TInvokeCause cause, TMSecTime nowTime) {
  
  /* Count working time. */
  static const char *label = "Take LogInfo";
  TElapsedTimer elapsedTime(label);
  
  /* Collect log. */
  return logManager->collectLog(jvmti, env, cause, nowTime);
}

/*!
 * \brief Interval collect log.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 * \param cause [in] Cause of taking a snapshot.<br>
 *                   This value is always Interval.
 */
void intervalLogProc(jvmtiEnv *jvmti, JNIEnv *env, TInvokeCause cause) {
  
  /* Call collect log by interval. */
  if (unlikely(!TakeLogInfo(jvmti, env, cause, (TMSecTime)getNowTimeSec()))) {
    PRINT_WARN_MSG("Failure interval collect log.");
  }
}

/*!
 * \brief Handle signal express user wanna collect normal log.
 * \param signo   [in] Number of received signal.
 * \param siginfo [in] Information of received signal.
 * \param data    [in] Data of received signal.
 */
void normalLogProc(int signo, void *siginfo, void *data) {
  
  /* Enable flag. */
  flagLogSignal = 1;
  NOTIFY_CATCH_SIGNAL;
}

/*!
 * \brief Handle signal express user wanna collect all log.
 * \param signo   [in] Number of received signal.
 * \param siginfo [in] Information of received signal.
 * \param data    [in] Data of received signal.
 */
void anotherLogProc(int signo, void *siginfo, void *data) {
  
  /* Enable flag. */
  flagAllLogSignal = 1;
  NOTIFY_CATCH_SIGNAL;
}

/*!
 * \brief Interval watching for log signal.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 */
void intervalSigProcForLog(jvmtiEnv *jvmti, JNIEnv *env) {
  
  /* If catch normal log signal. */
  if (unlikely(flagLogSignal != 0)) {
    TMSecTime nowTime = (TMSecTime)getNowTimeSec();
    flagLogSignal = 0;
    
    if (unlikely(!TakeLogInfo(jvmti, env, Signal, nowTime))) {
      PRINT_WARN_MSG("Failure collect log by normal log signal.");
    }
  }
  
  /* If catch all log signal. */
  if (unlikely(flagAllLogSignal != 0)) {
    TMSecTime nowTime = (TMSecTime)getNowTimeSec();
    flagAllLogSignal = 0;
    
    if (unlikely(!TakeLogInfo(jvmti, env, AnotherSignal, nowTime))) {
      PRINT_WARN_MSG("Failure collect log by all log signal.");
    }
  }
}

/*!
 * \brief Callback for deadlock trap and log.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 * \param cause [in] Cause of taking a snapshot.<br>
 *                   This value is always OccurredDeadlock.
 */
void onOccurredDeadLock(jvmtiEnv *jvmti, JNIEnv *env, TInvokeCause cause) {
  
  /* Get now date and time. */
  TMSecTime occurTime = lockFinder->getDeadlockTime();
  
  /* Collect log. */
  if (unlikely(!TakeLogInfo(jvmti, env, cause, occurTime))) {
    PRINT_WARN_MSG("Failure collect log on occurred deadlock.");
  }
}

/*!
 * \brief JVM resource exhausted event.
 * \param jvmti       [in] JVMTI environment object.
 * \param env         [in] JNI environment object.
 * \param flags       [in] Resource information bit flag.
 * \param reserved    [in] Reserved variable.
 * \param description [in] Description about resource exhausted.
 */
void JNICALL OnResourceExhausted(jvmtiEnv *jvmti, JNIEnv* env, jint flags,
  const void* reserved, const char* description) {
  
  /* Lock to use in multi-thread. */
  ENTER_PTHREAD_SECTION(&errMutex)
  
  /* If collected already and collect first only. */
  if (arg.firstCollect && arg.isFirstCollected) {
    
    /* Skip. */
    PRINT_WARN_MSG("Skip collect all log on JVM resource exhausted.");
    pthread_mutex_unlock(&errMutex);
    return;
  }
  arg.isFirstCollected = true;
  
  /* Unlock to use in multi-thread. */
  EXIT_PTHREAD_SECTION(&errMutex)
  
  /* Raise alert. */
  PRINT_CRIT_MSG_HEADER << "ALERT(RESOURCE): resource was exhausted."
                              << " info:\"" << description << "\"" << NEWLINE;
  
  /* Get now date and time. */
  TMSecTime nowTime = (TMSecTime)getNowTimeSec();
  
  if (arg.snmpSend) {
    /* Trap OID. */
    char trapOID[50]     = OID_RESALERT;
    oid OID_ALERT_DATE[] = {SNMP_OID_HEAPALERT, 1};
    oid OID_RES_FLAG[]   = {SNMP_OID_RESALERT,  1};
    oid OID_RES_DESC[]   = {SNMP_OID_RESALERT,  2};
    char buff[256] = {0};
    
    /* Send resource trap. */
    TTrapSender sender(SNMP_VERSION_2c, arg.snmpTarget, arg.snmpComName,
      162);
    
    /* Setting sysUpTime */
    sender.setSysUpTime();
    
    /* Setting trapOID. */
    sender.setTrapOID(trapOID);
    
    /* Set alert date. */
    snprintf(buff, 255, "%llu", nowTime);
    sender.addValue(OID_ALERT_DATE, OID_LENGTH(OID_ALERT_DATE),
                                             buff, SNMP_VAR_TYPE_COUNTER64);
    
    /* Set resource inforomation flags. */
    snprintf(buff, 255, "%d", flags);
    sender.addValue(OID_RES_FLAG, OID_LENGTH(OID_RES_FLAG),
                                             buff, SNMP_VAR_TYPE_INTEGER);
    
    /* Set resource decsription. */
    strncpy(buff, description, 255);
    sender.addValue(OID_RES_DESC, OID_LENGTH(OID_RES_DESC),
                                             buff, SNMP_VAR_TYPE_STRING);
    
    /* Send trap. */
    if (unlikely(sender.sendTrap() != SNMP_PROC_SUCCESS)) {
      /* Ouput message and clean up. */
      PRINT_WARN_MSG("Send SNMP resource exhausted trap failed!");
      sender.clearValues();
    }
  }
  
  /* Setting collect log cause. */
  TInvokeCause cause = ResourceExhausted;
  if (unlikely((flags & JVMTI_RESOURCE_EXHAUSTED_THREADS) > 0)) {
    cause = ThreadExhausted;
  }
  
  /* Collect log. */
  if (unlikely(!TakeLogInfo(jvmti, env, cause, nowTime))) {
    PRINT_WARN_MSG("Failure collect log on resource exhausted.");
  }
}

/*!
 * \brief JVM monitor contended entering event.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 * \param thread [in] Thread of monitor contended occurred.
 * \param object [in] Monitor object of thread contended.
 * \warning Sometimes this callback is called synchronizing or synchronized
 *          at safepoint.<br>Please be carefully calling inner function of JVM.
 */
void JNICALL OnMonitorContendedEnter(jvmtiEnv *jvmti, JNIEnv* env,
                                              jthread thread, jobject object) {
  
  /* Check deadlock. */
  TDeadlockList *list = NULL;
  int threadCnt = 0;
  char threadName[256] = {0};
  threadCnt = TDeadlockFinder::checkDeadlock(object, &list);
  
  /* If don't occur deadlock. */
  if (likely(threadCnt == 0)) {
    return;
  }
  
  { /* Get thread name. */
    jvmtiThreadInfo threadInfo = {0};
    jvmti->GetThreadInfo(thread, &threadInfo);
    strncpy(threadName, threadInfo.name, 255);
    
    /* Cleanup. */
    jvmti->Deallocate((unsigned char *)threadInfo.name);
    env->DeleteLocalRef(threadInfo.thread_group);
    env->DeleteLocalRef(threadInfo.context_class_loader);
  }
  
  /* Raise alert. */
  PRINT_CRIT_MSG_HEADER << "ALERT(DEADLOCK): occurred deadlock."
    << " threadCount:"  << threadCnt
    << " threadName:\"" << threadName << "\"" << NEWLINE;
  
  /* Print deadlock threads. */
  for (TDeadlockList *item = list; item != NULL; item = item->next) {
    
    /* Get thread information. */
    TJavaThreadInfo threadInfo = {0};
    getThreadDetailInfo(jvmti, env, item->thread, &threadInfo);
    
    PRINT_WARN_MSG_HEADER << "thread"
      << " name:\"" << threadInfo.name << "\""
      << " prio:"   << threadInfo.priority
      << NEWLINE;
    
    /* Cleanup. */
    free(threadInfo.name);
    free(threadInfo.state);
  }
  
  /* Cleanup deadlock thread list. */
  TDeadlockFinder::freeDeadlockList(list);
  
  /* Get now date and time. */
  TMSecTime nowTime = (TMSecTime)getNowTimeSec();
  if (arg.snmpSend) {
    /* Trap OID. */
    char trapOID[50]     = OID_DEADLOCKALERT;
    oid OID_ALERT_DATE[] = {SNMP_OID_HEAPALERT,     1};
    oid OID_DEAD_COUNT[] = {SNMP_OID_DEADLOCKALERT, 1};
    oid OID_DEAD_NAME[]  = {SNMP_OID_DEADLOCKALERT, 2};
    char buff[256] = {0};
    
    /* Send resource trap. */
    TTrapSender sender(SNMP_VERSION_2c, arg.snmpTarget, arg.snmpComName,
      162);
    
    /* Setting sysUpTime */
    sender.setSysUpTime();
    
    /* Setting trapOID. */
    sender.setTrapOID(trapOID);
    
    /* Set alert date. */
    snprintf(buff, 255, "%llu", nowTime);
    sender.addValue(OID_ALERT_DATE, OID_LENGTH(OID_ALERT_DATE), buff,
      SNMP_VAR_TYPE_COUNTER64);
    
    /* Set thread count. */
    snprintf(buff, 255, "%d", threadCnt);
    sender.addValue(OID_DEAD_COUNT, OID_LENGTH(OID_DEAD_COUNT), buff,
      SNMP_VAR_TYPE_COUNTER32);
    
    /* Set thread name. */
    sender.addValue(OID_DEAD_NAME, OID_LENGTH(OID_DEAD_NAME), threadName,
      SNMP_VAR_TYPE_STRING);
    
    /* Send trap. */
    if (unlikely(sender.sendTrap() != SNMP_PROC_SUCCESS)) {
      /* Ouput message and clean up. */
      PRINT_WARN_MSG("Send SNMP deadlock trap failed!");
      sender.clearValues();
    }
  }
  
  try {
    /* Sending notification means collect log by occurred deadlock. */
    lockFinder->notify(nowTime);
  } catch(...) {
    PRINT_WARN_MSG("Failure collect log on occurred deadlock.");
  }
}

/*!
 * \brief Setting JVMTI functional capabilities.
 * \param capabilities [out] Event capabilities structure.
 */
void setCapabilitiesForLog(jvmtiCapabilities *capabilities) {
  
  /* If collect log when JVM's resource exhausted. */
  if (arg.triggerOnLogError) {
    
    /* Enable heap and thread exhaustion event. */
    capabilities->can_generate_resource_exhaustion_heap_events = 1;
    capabilities->can_generate_resource_exhaustion_threads_events = 1;
  }
  
  /* If collect log when occurred deadlock. */
  if (arg.triggerOnLogLock) {
    capabilities->can_generate_monitor_events = 1;
  }
  
  /* Enable for original thread dump. */
  capabilities->can_get_source_file_name = 1;
  capabilities->can_get_line_numbers     = 1;
  capabilities->can_get_monitor_info     = 1;
  
  if (!isOnDemandAttached) {
    
    /*
     * Below capabilities cann't get,
     * if agent attached by on-demand-attach.
     * Because below two capability is able to get only JVM load phase.
     * At on-demand-attach, JVM is live phase or after phase.
     * 
     * See also
     *  openjdk/hotspot/src/share/vm/prims/jvmtiManageCapabilities.cpp
     */
    capabilities->can_get_owned_monitor_stack_depth_info = 1;
    capabilities->can_get_current_contended_monitor      = 1;
  }
}

/*!
 * \brief Setting enable of JVMTI and extension events for log function.
 * \param jvmti  [in] JVMTI environment object.
 * \param enable [in] Event notification is enable.
 * \return Setting process result.
 */
jint setEventEnableForLog(jvmtiEnv *jvmti, bool enable) {
  jvmtiEventMode mode = JVMTI_ENABLE;
  if (unlikely(!enable)) {
    mode = JVMTI_DISABLE;
  }
  
  /* If collect log when JVM's resource exhausted. */
  if (arg.triggerOnLogError) {
    
    /* Enable resource exhusted event. */
    SWITCH_EVENT_NOTIFICATION(jvmti, JVMTI_EVENT_RESOURCE_EXHAUSTED, mode)
  }
  
  /* If collect log when occurred deadlock. */
  if (arg.triggerOnLogLock) {
    
    /* Enable monitor contended event. */
    SWITCH_EVENT_NOTIFICATION(jvmti, JVMTI_EVENT_MONITOR_CONTENDED_ENTER, mode)
  }
  
  return SUCCESS;
}

/*!
 * \brief Setting enable of agent each threads for log function.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 * \param enable [in] Event notification is enable.
 */
void setThreadEnableForLog(jvmtiEnv *jvmti, JNIEnv *env, bool enable){
  /* Start or suspend HeapStats agent threads. */
  try {
    
    /* Switch interval log timer. */
    if (arg.LogInterval > 0) {
      if (enable) {
        logTimer->start(jvmti, env);
      } else {
        logTimer->stop();
      }
    }
    
    /* Reset signal flag even if non-processed signal is exist. */
    flagLogSignal    = 0;
    flagAllLogSignal = 0;
    
    if (likely(TDeadlockFinder::isCheckableDeadlock())) {
      
      if (arg.triggerOnLogLock) {
        /* Switch deadlock finder state. */
        if (enable) {
          lockFinder->start(jvmti, env);
        } else {
          lockFinder->stop();
        }
      }
    } else {
      
      /* Disable illegal flag. */
      arg.triggerOnLogLock = false;
    }
  } catch(const char *errMsg) {
    PRINT_WARN_MSG(errMsg);
  }
}

/*!
 * \brief JVM initialization event for log function.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 */
void onVMInitForLog(jvmtiEnv *jvmti, JNIEnv *env) {
  
  /* If failure jni archiver initialization. */
  if (unlikely(!TJniZipArchiver::globalInitialize(env))) {
    PRINT_WARN_MSG("Failure jni archiver initialization.");
  }
  
  /* Create instance for log function. */
  try{
    logManager = new TLogManager(env, jvmInfo);
  } catch(const char *errMsg) {
    PRINT_WARN_MSG(errMsg);
    arg.triggerOnLogSignal = false;
    arg.triggerOnLogLock   = false;
    arg.triggerOnLogError  = false;
    arg.LogInterval        = 0;
    
  } catch(...) {
    PRINT_WARN_MSG("AgentThread start failed!");
    arg.triggerOnLogSignal = false;
    arg.triggerOnLogLock   = false;
    arg.triggerOnLogError  = false;
    arg.LogInterval        = 0;
  }
  
  /* Initialize signal flag. */
  flagLogSignal    = 0;
  flagAllLogSignal = 0;
  
  /* Signal enable flag. */
  bool enableNormalSig = (arg.logSignalNormal != NULL)
                                    && (strlen(arg.logSignalNormal) > 0);
  bool enableAllSig    = (arg.logSignalAll    != NULL)
                                    && (strlen(arg.logSignalAll)    > 0);
  bool enableReloadSig = (arg.reloadSignal    != NULL)
                                    && (strlen(arg.reloadSignal)    > 0);
  
  /* If two signals is equal. */
  if (unlikely(enableNormalSig && enableAllSig
                       && strcmp(arg.logSignalNormal, arg.logSignalAll) == 0)) {
    
    /* Log signal is disable. */
    arg.triggerOnLogSignal = false;
    PRINT_WARN_MSG("Disabled conflict both log signal.");
    
  } else if (unlikely(enableNormalSig && enableReloadSig
    && strcmp(arg.logSignalNormal, arg.reloadSignal) == 0)) {
    
    /* Normal log signal is disable. */
    PRINT_WARN_MSG("Disabled conflict normal log signal.");
    free(arg.logSignalNormal);
    arg.logSignalNormal = strdup("");
    enableNormalSig = false;
    
  } else if (unlikely(enableAllSig && enableReloadSig
                           && strcmp(arg.logSignalAll, arg.reloadSignal) == 0)) {
    
    /* All log signal is disable. */
    PRINT_WARN_MSG("Disabled conflict all log signal.");
    free(arg.logSignalAll);
    arg.logSignalAll = strdup("");
    enableAllSig = false;
  }
  
  try {
    /* Create instance for log signal. */
    logSignalMngr    = new TSignalManager(env, &normalLogProc);
    logAllSignalMngr = new TSignalManager(env, &anotherLogProc);
    
    if (arg.triggerOnLogSignal) {
      /* If "normal log signal" is designated. */
      if (enableNormalSig) {
        
        /* Enable signal handling. */
        if (unlikely(!logSignalMngr->catchSignal(env, arg.logSignalNormal))) {
          
          PRINT_WARN_MSG("Failure register normal log signal.");
          /* Normal log signal is disable. */
          free(arg.logSignalNormal);
          arg.logSignalNormal = strdup("");
        }
      }
      
      /* If "all log signal" is designated. */
      if (enableAllSig) {
        
        /* Enable signal handling. */
        if (unlikely(!logAllSignalMngr->catchSignal(env, arg.logSignalAll))) {
          
          PRINT_WARN_MSG("Failure register all log signal.");
          /* All log signal is disable. */
          free(arg.logSignalAll);
          arg.logSignalAll = strdup("");
        }
      }
    }
  } catch(const char *errMsg) {
    PRINT_WARN_MSG(errMsg);
    arg.triggerOnLogSignal = false;
    
  } catch(...) {
    PRINT_WARN_MSG("Signal handler start failed!");
    arg.triggerOnLogSignal = false;
  }
}

/*!
 * \brief JVM finalization event for log function.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 */
void onVMDeathForLog(jvmtiEnv *jvmti, JNIEnv *env) {
  
  /* Flag of signal enable. */
  bool enableNormalSig = arg.attach && arg.triggerOnLogSignal
         && (arg.logSignalNormal != NULL) && (strlen(arg.logSignalNormal) != 0);
  bool enableAllSig = arg.attach && arg.triggerOnLogSignal
         && (arg.logSignalAll != NULL) && (strlen(arg.logSignalAll) != 0);
  
  /* If normal log signal is enabled. */
  if (likely(logSignalMngr != NULL)) {
    
    /* Terminate normal log signal manager. */
    if (enableNormalSig) {
      logSignalMngr->ignoreSignal(env, arg.logSignalNormal);
    }
    logSignalMngr->terminate(env);
  }
  delete logSignalMngr;
  logSignalMngr = NULL;
  
  /* If all log signal is enabled. */
  if (likely(logAllSignalMngr != NULL)) {
    
    /* Terminate all log signal manager. */
    if (enableAllSig) {
      logAllSignalMngr->ignoreSignal(env, arg.logSignalAll);
    }
    logAllSignalMngr->terminate(env);
  }
  delete logAllSignalMngr;
  logAllSignalMngr = NULL;
  
  /* Stop each thread. */
  logTimer->terminate();
  
  lockFinder->terminate();
  
  /* Destroy log manager. */
  delete logManager;
  logManager = NULL;
  
  /* Jni archiver finalization. */
  TJniZipArchiver::globalFinalize(env);
}

/*!
 * \brief Agent initialization for log function.
 * \param jvmti [in] JVMTI environment object.
 * \return Initialize process result.
 */
jint onAgentInitForLog(jvmtiEnv **jvmti) {
  /* Create thread instances that controlled log trigger. */
  try{
    if (unlikely(!TDeadlockFinder::globalInitialize())) {
      PRINT_WARN_MSG("Failure deadlock finder initialization");
      arg.triggerOnLogLock = false;
    }
    
    logTimer = new TTimer(&intervalLogProc,
                                 arg.LogInterval,  "HeapStats Log Timer");
    
    lockFinder = new TDeadlockFinder(&onOccurredDeadLock);
  } catch(const char *errMsg) {
    PRINT_CRIT_MSG(errMsg);
    return AGENT_THREAD_INITIALIZE_FAILED;
  } catch(...) {
    PRINT_CRIT_MSG("AgentThread initialize failed!");
    return AGENT_THREAD_INITIALIZE_FAILED;
  }
  
  return SUCCESS;
}

/*!
 * \brief Agent finalization for log function.
 */
void onAgentFinalForLog(void) {
  
  /* Destroy timer object. */
  delete logTimer;
  logTimer = NULL;
  
  /* Destroy deadlock finder object. */
  delete lockFinder;
  lockFinder = NULL;
}

