/*!
 * \file snapShotMain.hpp
 * \brief This file is used to take snapshot.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */

#ifndef _SNAPSHOT_MAIN_HPP
#define _SNAPSHOT_MAIN_HPP

#include <jni.h>
#include <jvmti.h>

#include "util.hpp"
#include "classContainer.hpp"
#include "snapShotProcessor.hpp"
#include "gcWatcher.hpp"
#include "timer.hpp"
#include "jvmInfo.hpp"

/* Define export function for calling from external. */
extern "C"{
  
  /*!
  * \brief Take a heap information snapshot.
  * \param jvmti [in] JVMTI environment object.
  * \param env   [in] JNI environment object.
  * \param cause [in] Cause of taking a snapshot.<br>
  *                   e.g. GC, DumpRequest or Interval.
  */
  void TakeSnapShot(jvmtiEnv *jvmti, JNIEnv *env, TInvokeCause cause);
};

/* Event for logging function. */

/*!
 * \brief Setting JVMTI functional capabilities.
 * \param capabilities [out] Event capabilities structure.
 */
void setCapabilitiesForSnapShot(jvmtiCapabilities *capabilities);

/*!
 * \brief Setting enable of JVMTI and extension events for snapshot function.
 * \param jvmti  [in] JVMTI environment object.
 * \param enable [in] Event notification is enable.
 * \return Setting process result.
 */
jint setEventEnableForSnapShot(jvmtiEnv *jvmti, bool enable);

/*!
 * \brief Setting enable of agent each threads for snapshot function.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 * \param enable [in] Event notification is enable.
 */
void setThreadEnableForSnapShot(jvmtiEnv *jvmti, JNIEnv *env, bool enable);

/*!
 * \brief JVM initialization event for snapshot function.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 */
void onVMInitForSnapShot(jvmtiEnv *jvmti, JNIEnv *env);

/*!
 * \brief JVM finalization event for snapshot function.
 * \param jvmti [in] JVMTI environment object.
 * \param env   [in] JNI environment object.
 */
void onVMDeathForSnapShot(jvmtiEnv *jvmti, JNIEnv *env);

/*!
 * \brief Agent initialization for snapshot function.
 * \param jvmti [in] JVMTI environment object.
 * \return Initialize process result.
 */
jint onAgentInitForSnapShot(jvmtiEnv **jvmti);

/*!
 * \brief Agent finalization for snapshot function.
 */
void onAgentFinalForSnapShot(void);

/* JVMTI events. */

/*!
 * \brief New class loaded event.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 * \param thread [in] Java thread object.
 * \param klass  [in] Newly loaded class object.
 */
void JNICALL OnClassPrepare(jvmtiEnv *jvmti, JNIEnv *env,
                                          jthread thread, jclass klass);

/*!
 * \brief Class unload event.
 * \param jvmti  [in] JVMTI environment object.
 * \param env    [in] JNI environment object.
 * \param thread [in] Java thread object.
 * \param klass  [in] Unload class object.
 * \sa    from: hotspot/src/share/vm/prims/jvmti.xml
 */
void JNICALL OnClassUnload(jvmtiEnv *jvmti, JNIEnv *env,
                                          jthread thread, jclass klass);

/*!
 * \brief Before garbage collection event.
 * \param jvmti [in] JVMTI environment object.
 */
void JNICALL OnGarbageCollectionStart(jvmtiEnv *jvmti);

/*!
 * \brief After garbage collection event.
 * \param jvmti [in] JVMTI environment object.
 */
void JNICALL OnGarbageCollectionFinish(jvmtiEnv *jvmti);

/*!
 * \brief Event of before garbage collection by CMS collector.
 * \param jvmti [in] JVMTI environment object.
 */
void JNICALL OnCMSGCStart(jvmtiEnv *jvmti);

/*!
 * \brief Event of after garbage collection by CMS collector.
 * \param jvmti [in] JVMTI environment object.
 */
void JNICALL OnCMSGCFinish(jvmtiEnv *jvmti);

/*!
 * \brief Data dump request event.
 * \param jvmti [in] JVMTI environment object.
 */
void JNICALL OnDataDumpRequest(jvmtiEnv *jvmti);

/* Snapshot components. */

extern TClassContainer *clsContainer;
extern TSnapShotProcessor *snapShotProcessor;
extern TGCWatcher *gcWatcher;
extern TTimer *timer;
extern TSnapShotContainer *snapshotByGC;

#endif // _SNAPSHOT_MAIN_HPP
