/*
 * Launcher.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats;

import java.io.File;
import java.io.FileFilter;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import jp.co.ntt.oss.heapstats.gui.GuiCommon;

/**
 * Load the classes needed to start the application heapstats.
 */
public final class Launcher {
    /**
     * Don't let anyone instantiate this class.
     */
    private Launcher() {
    }

    /**
     * Return the Class load files.
     *
     * @param libDir library path
     * @param pattern The expression to be compiled
     * @return Return the Class load files.
     */
    private static File[] getJarList(final File libDir, final String pattern) {
        File[] ret = libDir.listFiles(new FileFilter() {
            private Pattern pat = Pattern.compile(pattern);

            public boolean accept(final File pathname) {
                return this.pat.matcher(pathname.getName()).matches();
            }
        });

        if ((ret == null) || (ret.length < 1)) {
            String errMsg = GuiCommon.getResources().getString(
                    GuiCommon.BASEE09);

            System.err.println(errMsg);
            JOptionPane.showMessageDialog(null, errMsg.replace("%s", pattern),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
        }

        return ret;
    }

    /**
     * Return the URLs from which to load classes and resources.
     *
     * @param files library paths list
     * @return Return the URLs from which to load classes and resources
     * @throws MalformedURLException If a protocol handler for the URL could not
     *         be found, or if some other error occurred while constructing the
     *         URL
     */
    private static URL[] buildURLArray(final Object... files)
            throws MalformedURLException {
        ArrayList<URL> ary = new ArrayList<URL>();

        for (Object obj : files) {
            if (obj instanceof File[]) {
                for (File f : (File[]) obj) {
                    ary.add(f.toURI().toURL());
                }
            } else if (obj instanceof File) {
                ary.add(((File) obj).toURI().toURL());
            }
        }

        return ary.toArray(new URL[0]);
    }

    /**
     * Starting to prepare for HeapStats Analyzer.
     *
     * @param args the arguments used for the method call
     */
    public static void main(final String[] args) {
        try {
            GuiCommon.load();
        } catch (Exception e) {
            GuiCommon.showMessage(null, GuiCommon.BASEW01,
                    JOptionPane.WARNING_MESSAGE);
        } catch (Throwable e) {
            GuiCommon.showMessage(null, GuiCommon.BASEW01,
                    JOptionPane.WARNING_MESSAGE);
        }

        try {
            // Get original JAR (heapstats.jar)
            File appJarFile = new File(System.getProperty("java.class.path"));
            // Get library directory path
            String libDirName = appJarFile.getParent();
            File libDir;
            if (libDirName == null) {
                libDir = new File("lib");
            } else {
                libDir = new File(libDirName + File.separator + "lib");
            }

            if (!libDir.isDirectory()) {
                String errMsg = GuiCommon.getResources().getString(
                        GuiCommon.BASEE09);

                JOptionPane.showMessageDialog(null,
                        errMsg.replace("%s", libDir.getPath()),
                        GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);

                return;
            }

            // Get jcommon library jar
            File[] jcommons = getJarList(libDir,
                    "^jcommon(\\-[\\d\\.]+)?\\.jar$");
            // Get JFreeChart library jar
            File[] jfrees = getJarList(libDir,
                    "^jfreechart(\\-[\\d\\.]+)?\\.jar$");

            if (jcommons != null && jcommons.length > 0 && jfrees != null
                    && jfrees.length > 0) {
                // Create class loader
                ClassLoader clsLdr = new LaunchClassLoader(buildURLArray(
                        jcommons, jfrees, appJarFile));

                // Load "GUIMain"
                Class<?> cls = clsLdr
                        .loadClass("jp.co.ntt.oss.heapstats.gui.GUIMain");

                // invoke "main" method
                Method method = cls.getMethod("main",
                        new Class[] {String[].class});
                method.invoke(null, new Object[] {args});
            }
        } catch (Exception e) {
            GuiCommon.showMessage(null, GuiCommon.BASEE16,
                    JOptionPane.ERROR_MESSAGE);
        } catch (Throwable e) {
            GuiCommon.showMessage(null, GuiCommon.BASEE16,
                    JOptionPane.ERROR_MESSAGE);
        }
    }
}
