/*
 * CSVDumper.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.csv;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import jp.co.ntt.oss.heapstats.container.MapKey;
import jp.co.ntt.oss.heapstats.container.ObjectData;
import jp.co.ntt.oss.heapstats.container.SnapShot;
import jp.co.ntt.oss.heapstats.container.SnapShotHeader;
import jp.co.ntt.oss.heapstats.gui.DiffDialog;
import jp.co.ntt.oss.heapstats.gui.GuiCommon;
import jp.co.ntt.oss.heapstats.gui.HeapProgressBar;
import jp.co.ntt.oss.heapstats.gui.snapshot.SnapShotReader;

/**
 * extends {@link SwingWorker}<br>
 * implements {@link PropertyChangeListener}<br>
 * <br>
 * Output to a CSV file or GC heap usage statistics.
 */
public class CSVDumper extends SwingWorker<String, String> implements
                                                          PropertyChangeListener {
    /** The format of a CSV snapshot to date. */
    private static final String SNAPSHOT_DATE_FORMAT = "yyyyMMdd HH:mm:ss.SSS";

    /** progress bar. */
    private HeapProgressBar progressBar;

    /** output object list. */
    private List<List<Object>> diffOffsets;

    /** Save the file name. */
    private String csvFile;

    /** Delta Index chart, draw a stacked heap usage chart. */
    private DiffDialog deltaDialog;

    /** dump type. true:heap usage. */
    private boolean dumpType;

    /** parent Frame. */
    private JFrame owner;

    /**
     * Constructs a CSVDumper object.
     *
     * @param parent determines the Frame in which the dialog is displayed; if
     *        null, or if the parentComponent has no Frame, a default Frame is
     *        used
     */
    public CSVDumper(final JFrame parent) {
        diffOffsets = null;
        deltaDialog = null;
        owner = parent;
        addPropertyChangeListener(this);
    }

    /**
     * Sets the object lists.
     *
     * @param objectLists output object list
     */
    public final void setDiffOffsets(final List<List<Object>> objectLists) {
        diffOffsets = objectLists;
    }

    /**
     * Sets the save file name.
     *
     * @param fname CSV output file name
     */
    public final void setCsvFile(final String fname) {
        csvFile = fname;
    }

    /**
     * Sets the Delta dialog.
     *
     * @param dialog the Delta dialog
     */
    public final void setDelta(DiffDialog dialog) {
        deltaDialog = dialog;
    }

    /**
     * Sets the dump type.
     *
     * @param type true:java heap csv file , false:gc statistics file
     */
    public final void setDumpType(final boolean type) {
        dumpType = type;
    }

    /**
     * Filter the objects and returns the results. true:Exclude external
     *
     * @param elmt ObjectData
     * @return Return the Check result
     */
    private boolean checkFilter(final ObjectData elmt) {
        boolean result = false;

        // Start Filter
        boolean match = false;
        for (Pattern ptn : deltaDialog.getFilterList()) {
            Matcher matcher = ptn.matcher(elmt.getName());
            if (matcher.matches()) {
                match = true;
                break;
            }
        }

        if (!match) {
            // Matches or, when there is no filter
            result = true;
        }

        // Class Search
        Object[] searchClass = deltaDialog.getSearchClass();
        if (searchClass != null && searchClass.length > 0 && result) {
            for (Object path : searchClass) {
                if (elmt.getName().equals(path)) {
                    result = true;
                    break;
                }
                result = false;
            }
        }

        return result;
    }

    /**
     * Output to a CSV file for each snapshot of the number of instances of
     * objects and heap usage.
     */
    private void dumpHeapCSV() {
        PrintWriter writer = null;
        Map<MapKey, long[]> table = new HashMap<MapKey, long[]>();

        try {
            if (!csvFile.toLowerCase().endsWith(".csv")) {
                writer = new PrintWriter(new FileWriter(csvFile + ".csv"));
            } else {
                writer = new PrintWriter(new FileWriter(csvFile));
            }

            SimpleDateFormat sdf = new SimpleDateFormat(SNAPSHOT_DATE_FORMAT);
            writer.print("Tag, Name");

            for (int i = 0; i < diffOffsets.size(); i++) {
                List<Object> data = diffOffsets.get(i);
                SnapShot snapshot = ((SnapShotReader) data.get(1))
                        .getSnapShot((Long) data.get(0));

                /* Write CSV Header */
                writer.print(", ");
                writer.print(sdf.format(snapshot.getHeader().getSnapShotDate()));
                writer.print("_instances, ");
                writer.print(sdf.format(snapshot.getHeader().getSnapShotDate()));
                writer.print("_total size");

                /* Write Data */
                for (Map.Entry<Long, ObjectData> entry : snapshot.getElements()
                        .entrySet()) {
                    MapKey key = new MapKey(entry.getValue());
                    long[] tmp = table.get(key);
                    if (checkFilter(entry.getValue())) {
                        if (tmp == null) {
                            long[] row = new long[diffOffsets.size() * 2];
                            row[i * 2] = entry.getValue().getCount();
                            row[i * 2 + 1] = entry.getValue().getTotalSize();
                            table.put(key, row);
                        } else {
                            tmp[i * 2] = entry.getValue().getCount();
                            tmp[i * 2 + 1] = entry.getValue().getTotalSize();
                            table.put(key, tmp);
                        }
                    }
                }
                setProgress(90 * (i + 1) / diffOffsets.size());
            }

            writer.println();

            int i = 1;
            for (Map.Entry<MapKey, long[]> entry : table.entrySet()) {
                writer.print("0x");
                writer.print(Long.toHexString(entry.getKey().getTag()));
                writer.print(", ");
                writer.print(entry.getKey().getName());

                for (long row : entry.getValue()) {
                    writer.print(", ");
                    writer.print(row);
                }

                writer.println();
                setProgress(90 + 10 * (i++) / table.size());
            }

            table.clear();

            GuiCommon.showMessage(owner, GuiCommon.BASEI02,
                    JOptionPane.INFORMATION_MESSAGE);
        } catch (IOException e) {
            JOptionPane.showMessageDialog(owner,
                    GuiCommon.getResources().getString(GuiCommon.BASEE15)
                            .replace("%s", e.getLocalizedMessage()),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            JOptionPane.showMessageDialog(owner,
                    GuiCommon.getResources().getString(GuiCommon.BASEE15)
                            .replace("%s", e.getLocalizedMessage()),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        } catch (Exception e) {
            String msg = e.getLocalizedMessage();
            if (msg == null) {
                msg = e.toString();
            }
            JOptionPane.showMessageDialog(owner, GuiCommon.getResources()
                    .getString(GuiCommon.BASEE15).replace("%s", msg),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }

    /**
     * Output to a CSV file for each snapshot statistics for GC.
     */
    private void dumpGcCSV() {
        PrintWriter writer = null;
        try {
            if (!csvFile.toLowerCase().endsWith(".csv")) {
                writer = new PrintWriter(new FileWriter(csvFile + ".csv"));
            } else {
                writer = new PrintWriter(new FileWriter(csvFile));
            }

            SimpleDateFormat sdf = new SimpleDateFormat(SNAPSHOT_DATE_FORMAT);

            writer.print("SnapShot Date,");
            writer.print("Full GC Count,");
            writer.print("Young GC Count,");
            writer.print("New Heap Usage,");
            writer.print("Old Heap Usage,");
            writer.print("Total Heap Size,");
            writer.print("GC Cause,");
            writer.print("GC Time");
            writer.println();

            int i = 1;
            for (List<Object> data : diffOffsets) {
                SnapShot snapshot = ((SnapShotReader) data.get(1))
                        .getSnapShot((Long) data.get(0));
                SnapShotHeader header = snapshot.getHeader();

                writer.print(sdf.format(header.getSnapShotDate()));
                writer.print(",");
                writer.print(header.getFullCount());
                writer.print(",");
                writer.print(header.getYngCount());
                writer.print(",");
                writer.print(header.getNewHeap());
                writer.print(",");
                writer.print(header.getOldHeap());
                writer.print(",");
                writer.print(header.getTotalSize());
                writer.print(",");
                writer.print(header.getGcCause());
                writer.print(",");
                if (header.getGcTime() == 0) {
                    writer.print("-");
                } else {
                    writer.print(header.getGcTime());
                }
                writer.println();
                setProgress(100 * i++ / diffOffsets.size());
            }

            GuiCommon.showMessage(owner, GuiCommon.BASEI02,
                    JOptionPane.INFORMATION_MESSAGE);
        } catch (IOException e) {
            JOptionPane.showMessageDialog(owner,
                    GuiCommon.getResources().getString(GuiCommon.BASEE15)
                            .replace("%s", e.getLocalizedMessage()),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            JOptionPane.showMessageDialog(owner,
                    GuiCommon.getResources().getString(GuiCommon.BASEE15)
                            .replace("%s", e.getLocalizedMessage()),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        } catch (Exception e) {
            String msg = e.getLocalizedMessage();
            if (msg == null) {
                msg = e.toString();
            }
            JOptionPane.showMessageDialog(owner, GuiCommon.getResources()
                    .getString(GuiCommon.BASEE15).replace("%s", msg),
                    GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }

    @Override
    public final void propertyChange(final PropertyChangeEvent evt) {
        String strPropertyName = evt.getPropertyName();
        if ("progress".equals(strPropertyName)) {
            int progress = (Integer) evt.getNewValue();
            progressBar.setValue(progress);
        }
    }

    @Override
    protected final String doInBackground() throws Exception {
        progressBar = new HeapProgressBar("CSV dump start...", owner);
        progressBar.setVisible(true);
        if (dumpType) {
            dumpHeapCSV();
        } else {
            dumpGcCSV();
        }
        return null;
    }

    @Override
    protected final void done() {
        super.done();
        progressBar.setVisible(false);
    }
}
