/*
 * ResourceDialog.java
 * Created on 2012/01/20
 * RCSID: $Id: ResourceDialog.java 407 2012-05-21 00:30:29Z alpha_takahiro.jyoukou $
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.table.DefaultTableModel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryMarker;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.DatasetRenderingOrder;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.LineAndShapeRenderer;
import org.jfree.data.category.DefaultCategoryDataset;

/**
 * extends {@link JDialog}<br>
 * implements {@link ActionListener}<br>
 * <br>
 * Draw a graphical representation of the java process resources.
 */
public class ResourceDialog extends JDialog implements
                                           ActionListener {

    /** serialVersionUID. */
    private static final long serialVersionUID = -2143023955517445278L;

    /** Mainframe heapstats. */
    private GUIMain guimain;

    /** Angle of the axis labels. */
    private static final double ANGLE = 6.0;

    /** The index of the collection date in csv file(Collection time). */
    private static final int COLLECT_DATE = 0;

    /** The index of the collection date in csv file(java user time). */
    private static final int JAVA_CPU_USER = 2;

    /** The index of the collection date in csv file(java system time). */
    private static final int JAVA_CPU_SYSTEM = 3;

    /** The index of the collection date in csv file(VSZ). */
    private static final int JAVA_VSZ = 4;

    /** The index of the collection date in csv file(RSS). */
    private static final int JAVA_RSS = 5;

    /** The index of the collection date in csv file(user time). */
    private static final int CPU_USER = 6;

    /** The index of the collection date in csv file(user time nice). */
    private static final int CPU_USER_NICE = 7;

    /** The index of the collection date in csv file(system time). */
    private static final int CPU_SYSTEM = 8;

    /** The index of the collection date in csv file(idle task). */
    private static final int CPU_IDLE = 9;

    /** The index of the collection date in csv file(iowait ). */
    private static final int CPU_IO = 10;

    /** The index of the collection date in csv file(Interrupt ReQuest). */
    private static final int CPU_IRQ = 11;

    /** The index of the collection date in csv file(Soft Interrupt ReQuest). */
    private static final int CPU_SOFT_IRQ = 12;

    /** The index of the collection date in csv file(CPU steal). */
    private static final int CPU_STEAL = 13;

    /** The index of the collection date in csv file(CPU Guest). */
    private static final int CPU_GUEST = 14;

    /**
     * The index of the collection date in csv file(Conflict Monitor
     * occurrences).
     */
    private static final int CONFLICT_MONITOR = 15;

    /** The index of the collection date in csv file(safepointtime). */
    private static final int SAFE_POINT_TIME = 16;

    /** The index of the collection date in csv file(safepoints). */
    private static final int SAFE_POINTS = 17;

    /** The index of the collection date in csv file(threads live). */
    private static final int THREADS_LIVE = 18;

    /** The index of the collection date in csv file(file name). */
    private static final int ARCHIVE_FILE = 19;

    /** LogDetail Data. */
    private LogDetailDialog detailDlg;

    /** Main Component. */
    private Box comp = Box.createVerticalBox();

    /** Check box to set whether the graph of JavaCPU. */
    private JCheckBox javaCpu;

    /** Check box to set whether the graph of systemCPU. */
    private JCheckBox sysCpu;

    /** Check box to set whether the graph of JavaMem. */
    private JCheckBox javaMem;

    /** Check box to set whether the graph of threads. */
    private JCheckBox threads;

    /** Split pane at the top of the resource. */
    private JSplitPane top;

    /** Split pane at the bottom of the resource. */
    private JSplitPane bottom;

    /** Vertical split pane Resources. */
    private JSplitPane center;

    /** Chart panel JavaCPU. */
    private ChartPanel javaCPUchart;

    /** Chart panel SystemCPU. */
    private ChartPanel sysCPUchart;

    /** Chart panel JavaMemory. */
    private ChartPanel memChart;

    /** Chart panel threads. */
    private ChartPanel threadChart;

    /** Summary table model for the process display. */
    private DefaultTableModel model;

    /** The SwingWorker for drawing graphs. */
    private ResouceWorker worker;

    /**
     * Creates a modeless dialog with the specified Dialog as its owner.
     *
     * @param owner the owner Dialog from which the dialog is displayed or null
     *        if this dialog has no owner
     * @param dialog the LogDetail dialog
     */
    public ResourceDialog(final GUIMain owner, final LogDetailDialog dialog) {
        guimain = owner;
        detailDlg = dialog;

        setTitle(GuiCommon.getResources()
                .getString(GuiCommon.TAB_TITLE_RESOUCE));
        setSize(GuiCommon.DEFAULT_DIALOG_WIDTH, GuiCommon.DEFAULT_DIALOG_HEIGHT);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setModal(false);

        ChartFactory.setChartTheme(StandardChartTheme.createLegacyTheme());

        comp.setBorder(BorderFactory.createEmptyBorder(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE, GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));
        setLocationByPlatform(true);
    }

    /**
     * Return the component.
     *
     * @return Return the Component
     */
    public final Box getComp() {
        return comp;
    }

    /**
     * Sets the dialog component.
     *
     * @param value the Main Component
     */
    public final void setComp(final Box value) {
        comp = value;
        add(comp);
    }

    /**
     * Starts drawing the Resource Data.
     *
     * @param files csv list of selected files
     * @param dataMap Key is Date. Data is the index of variable files & combo
     *        box.
     * @param startIndex Starting date of the drawing area
     * @param endIndex Ending date of the drawing area
     */
    public final void draw(final List<File> files,
            final Map<String, Integer[]> dataMap, final int startIndex,
            final int endIndex) {
        clear();
        addComponent();

        int idx = 0;
        Map<File, List<Integer>> rowIndexes = new HashMap<File, List<Integer>>();
        List<File> fileList = new ArrayList<File>();
        for (Map.Entry<String, Integer[]> entry : dataMap.entrySet()) {
            if (idx >= startIndex && idx <= endIndex) {
                Integer[] data = entry.getValue();
                File file = files.get(data[0]);

                List<Integer> rowList = rowIndexes.get(file);
                if (rowList == null) {
                    rowList = new ArrayList<Integer>();
                    fileList.add(file);
                }

                rowList.add(data[1]);
                rowIndexes.put(file, rowList);
            }
            idx++;
        }

        if (rowIndexes.size() > 0) {

            worker = new ResouceWorker(this, rowIndexes, fileList,
                    (endIndex + 1) - startIndex);
            worker.execute();
        }
    }

    /**
     * Returns true if this task completed. Completion may be due to normal
     * termination, an exception, or cancellation -- in all of these cases, this
     * method will return true.
     *
     * @return true if this task completed
     */
    public final boolean isDone() {
        if (worker != null) {
            return worker.isDone();
        }
        return true;
    }

    @Override
    public final void actionPerformed(final ActionEvent actionevent) {
        Object eventSource = actionevent.getSource();

        if (eventSource.equals(javaCpu)) {
            if (javaCpu.isSelected()) {
                top.setLeftComponent(javaCPUchart);
            } else {
                top.setLeftComponent(null);
            }
            if (!javaCpu.isSelected() && !sysCpu.isSelected()) {
                center.setTopComponent(null);
            } else {
                center.setTopComponent(top);
            }
        } else if (eventSource.equals(sysCpu)) {
            if (sysCpu.isSelected()) {
                top.setRightComponent(sysCPUchart);
            } else {
                top.setRightComponent(null);
            }
            if (!javaCpu.isSelected() && !sysCpu.isSelected()) {
                center.setTopComponent(null);
            } else {
                center.setTopComponent(top);
            }
        } else if (eventSource.equals(javaMem)) {
            if (javaMem.isSelected()) {
                bottom.setLeftComponent(memChart);
            } else {
                bottom.setLeftComponent(null);
            }
            if (!javaMem.isSelected() && !threads.isSelected()) {
                center.setBottomComponent(null);
            } else {
                center.setBottomComponent(bottom);
            }
        } else if (eventSource.equals(threads)) {
            if (threads.isSelected()) {
                bottom.setRightComponent(threadChart);
            } else {
                bottom.setRightComponent(null);
            }
            if (!javaMem.isSelected() && !threads.isSelected()) {
                center.setBottomComponent(null);
            } else {
                center.setBottomComponent(bottom);
            }
        }
    }

    /**
     * Create components to draw on the screen.
     */
    private void addComponent() {
        comp.add(addGraphPanel());
        comp.revalidate();
        comp.add(Box.createVerticalStrut(GuiCommon.BORDER_SIZE));
        comp.add(addSummaryPanel());
        comp.revalidate();
    }

    /**
     * Create a component of the process to return the report.
     *
     * @return Return the accordion panel component
     */
    private ExpansionContractionPanel addSummaryPanel() {
        ExpansionContractionPanel threadSummary = new ExpansionContractionPanel(
                GuiCommon.getResources().getString(
                        GuiCommon.TITLE_THREAD_SUMMARY));

        model = new DefaultTableModel();
        model.setColumnCount(2);
        model.setRowCount(0);
        JTable entries = new JTable(model);

        threadSummary.setComponent(entries, false);

        return threadSummary;
    }

    /**
     * Create a component of the resource to return the report.
     *
     * @return Return the accordion panel component
     */
    private ExpansionContractionPanel addGraphPanel() {
        ExpansionContractionPanel graphMain = new ExpansionContractionPanel(
                GuiCommon.getResources().getString(
                        GuiCommon.TITLE_RESOURCE_REPORT));
        JPanel graphPanel = new JPanel(new BorderLayout());

        JPanel checkPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        checkPanel.setBackground(Color.WHITE);
        javaCpu = new JCheckBox(GuiCommon.getResources().getString(
                GuiCommon.GRAPH_JAVA_CPU), true);
        sysCpu = new JCheckBox(GuiCommon.getResources().getString(
                GuiCommon.GRAPH_SYSTEM_CPU), true);
        javaMem = new JCheckBox(GuiCommon.getResources().getString(
                GuiCommon.GRAPH_JAVA_MEMORY), true);
        threads = new JCheckBox(GuiCommon.getResources().getString(
                GuiCommon.GRAPH_THREADS), true);

        javaCpu.setBackground(Color.WHITE);
        sysCpu.setBackground(Color.WHITE);
        javaMem.setBackground(Color.WHITE);
        threads.setBackground(Color.WHITE);

        javaCpu.addActionListener(this);
        sysCpu.addActionListener(this);
        javaMem.addActionListener(this);
        threads.addActionListener(this);

        checkPanel.add(javaCpu);
        checkPanel.add(sysCpu);
        checkPanel.add(javaMem);
        checkPanel.add(threads);

        top = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        bottom = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        center = new JSplitPane(JSplitPane.VERTICAL_SPLIT, top, bottom);

        top.setDividerSize(5);
        bottom.setDividerSize(5);
        center.setDividerSize(5);

        top.setResizeWeight(0.5);
        bottom.setResizeWeight(0.5);
        center.setResizeWeight(0.5);

        graphPanel.add(checkPanel, BorderLayout.NORTH);
        graphPanel.add(center, BorderLayout.CENTER);

        graphMain.setComponent(graphPanel, true);

        return graphMain;
    }

    /**
     * Clear the resource data.
     */
    public final void clear() {
        for (Component obj : comp.getComponents()) {
            comp.remove(obj);
        }
        comp.setVisible(false);

        detailDlg.clear();
    }

    /**
     * extends {@link SwingWorker}<br>
     * <br>
     * SwingWorker thread drawing done in the time-consuming process chart.
     */
    private class ResouceWorker extends SwingWorker<Object, Object> {
        /** the alpha transparency (must be in the range 0.0f to 1.0f). */
        private static final float ALPHA_TRANSPARENCY = 0.7f;

        /** ResourceDialog Data. */
        private ResourceDialog resourceDlg;

        /** rowindex list. */
        private Map<File, List<Integer>> rowIndexes;

        /** CSV file an ordered list of reading. */
        private List<File> fileList;

        /** Chart data sets javaCPU. */
        private DefaultCategoryDataset javaCpu;

        /** Chart data sets SystemCPU. */
        private DefaultCategoryDataset systemCpu;

        /** Chart data sets javaMemory. */
        private DefaultCategoryDataset javaMem;

        /** data sets threads,monitor and safepoints. */
        private DefaultCategoryDataset threads;

        /** data sets safepointtime. */
        private DefaultCategoryDataset safeTime;

        /** The number of data to be rendered. */
        private int rowCount;

        /** ProgressBar. */
        private HeapProgressBar progressBar;

        /** The format of the time you draw on screen. */
        public static final String CHART_DATE_FORMAT = "MMdd HH:mm:ss";

        /***
         * Create a SwingWorker.
         *
         * @param dialog Resource Data
         * @param indexes rowindex list
         * @param files CSV file to be read list
         * @param count The number of data to be rendered
         */
        public ResouceWorker(final ResourceDialog dialog,
                final Map<File, List<Integer>> indexes, final List<File> files,
                final int count) {
            resourceDlg = dialog;
            rowIndexes = indexes;
            fileList = files;
            rowCount = count;

            // The resource of a graph is initialized.
            initJavaCPU();
            initSystemCPU();
            initJavaMemory();
            initThreads();
        }

        @Override
        protected Object doInBackground() throws Exception {
            List<String> zipFile = new ArrayList<String>();
            List<String> hideCol = new ArrayList<String>();
            SimpleDateFormat sdf = new SimpleDateFormat(CHART_DATE_FORMAT);
            progressBar = new HeapProgressBar("Draw the 'Log Collect'.",
                    guimain);
            progressBar.setVisible(true);

            try {
                float cpuAve = 0;
                float cpuPeak = 0;
                long vszAve = 0;
                long vszPeak = 0;
                long rssAve = 0;
                long rssPeak = 0;

                int count = 0;

                long[] last = null;
                for (File file : fileList) {
                    BufferedReader read = new BufferedReader(new FileReader(
                            file));

                    List<Integer> range = rowIndexes.get(file);
                    String line = read.readLine();
                    int idx = 0;
                    while (line != null) {
                        String[] tmp = line.split(",");
                        long[] current = stringToLong(tmp);

                        if (range.contains(idx)) {
                            if (last != null) {
                                // Java CPU
                                drawJavaCPU(current, last, tmp[ARCHIVE_FILE]
                                        .trim().length());
                                drawSystemCPU(current, last, tmp[ARCHIVE_FILE]
                                        .trim().length());
                                drawThreads(current, last, tmp[ARCHIVE_FILE]
                                        .trim().length());
                                float cpu = getCpuUtilization(current, last);
                                cpuAve += cpu;
                                cpuPeak = checkPeak(cpuPeak, cpu);
                            }
                            drawJavaMemory(current, tmp[ARCHIVE_FILE].trim()
                                    .length());

                            vszAve += current[JAVA_VSZ];
                            vszPeak = (long) checkPeak(vszPeak,
                                    current[JAVA_VSZ]);
                            rssAve += current[JAVA_RSS];
                            rssPeak = (long) checkPeak(rssPeak,
                                    current[JAVA_RSS]);

                            if (tmp[ARCHIVE_FILE].trim().length() > 0) {
                                zipFile.add(tmp[ARCHIVE_FILE + 1]
                                        + File.separator + tmp[ARCHIVE_FILE]);
                            }

                            if (((rowCount / 10) > 0)
                                    && (count % (rowCount / 10) != 0)) {
                                hideCol.add(sdf.format(current[COLLECT_DATE]));
                            }

                            count++;

                            float value = (float) count / (float) rowCount
                                    * 100;
                            progressBar.setValue((int) value);
                        }

                        last = current.clone();

                        // One line is read.
                        line = read.readLine();

                        idx++;
                    }

                    read.close();
                }

                hideCategoryLabel(javaCPUchart.getChart().getCategoryPlot()
                        .getDomainAxis(), hideCol);
                hideCategoryLabel(sysCPUchart.getChart().getCategoryPlot()
                        .getDomainAxis(), hideCol);
                hideCategoryLabel(memChart.getChart().getCategoryPlot()
                        .getDomainAxis(), hideCol);
                hideCategoryLabel(threadChart.getChart().getCategoryPlot()
                        .getDomainAxis(), hideCol);

                createTable(cpuAve, cpuPeak, vszAve, vszPeak, rssAve, rssPeak,
                        count);

                detailDlg.draw(zipFile, 0);

            } catch (Throwable e) {
                GuiCommon.showMessage(resourceDlg, GuiCommon.BASEE20,
                        JOptionPane.ERROR_MESSAGE);
                e.printStackTrace();
            }

            return null;
        }

        @Override
        protected void done() {
            super.done();
            progressBar.setVisible(false);
            if (resourceDlg.isVisible()) {
                comp.setVisible(true);
            }
            comp.revalidate();

            resourceDlg.validate();
        }

        /**
         * Convert a string to Long.
         *
         * @param row A list of one row of data in a csv file
         * @return Return the Long list
         */
        private long[] stringToLong(final String[] row) {
            long[] ret = new long[row.length];

            for (int i = 0; i < row.length; i++) {
                if (!row[i].isEmpty() && row[i].matches("\\d+")) {
                    ret[i] = Long.parseLong(row[i]);
                }
            }

            return ret;
        }

        /**
         * Returns the system utilization cpu.
         *
         * @param current Current data obtained from csv
         * @param last csv data obtained from previous
         * @return Returns the system utilization cpu.
         */
        private long getCpuUtilization(final long[] current, final long[] last) {
            float total = (current[CPU_USER] - last[CPU_USER])
                    + (current[CPU_USER_NICE] - last[CPU_USER_NICE])
                    + (current[CPU_SYSTEM] - last[CPU_SYSTEM])
                    + (current[CPU_IDLE] - last[CPU_IDLE])
                    + (current[CPU_IO] - last[CPU_IO])
                    + (current[CPU_IRQ] - last[CPU_IRQ])
                    + (current[CPU_SOFT_IRQ] - last[CPU_SOFT_IRQ])
                    + (current[CPU_STEAL] - last[CPU_STEAL])
                    + (current[CPU_GUEST] - last[CPU_GUEST]);
            float java = (current[JAVA_CPU_SYSTEM] - last[JAVA_CPU_SYSTEM])
                    + (current[JAVA_CPU_USER] - last[JAVA_CPU_USER]);

            return (long) ((java / total) * 100);
        }

        /**
         * Create a summary table of the process.
         *
         * @param cpuAve Average CPU utilization
         * @param cpuPeak Peak CPU utilization
         * @param vszAve Average virtual memory usage
         * @param vszPeak Peak virtual memory usage
         * @param rssAve Average physical memory usage
         * @param rssPeak Peak physical memory usage
         * @param count Data count
         */
        private void createTable(final float cpuAve, final float cpuPeak,
                final long vszAve, final long vszPeak, final long rssAve,
                final long rssPeak, final int count) {
            Object[] row = new Object[2];

            NumberFormat nbf = NumberFormat.getIntegerInstance();

            row[0] = GuiCommon.getResources().getObject(
                    GuiCommon.ITEM_CPU_AVERAGE);
            row[1] = cpuAve / (count - 1) + " %";
            model.addRow(row);
            row[0] = GuiCommon.getResources()
                    .getObject(GuiCommon.ITEM_CPU_PEAK);
            row[1] = cpuPeak + " %";
            model.addRow(row);
            row[0] = GuiCommon.getResources().getObject(
                    GuiCommon.ITEM_VSZ_AVERAGE);
            row[1] = nbf.format(vszAve / count / GuiCommon.BINARY_PREFIX)
                    + " KB";
            model.addRow(row);
            row[0] = GuiCommon.getResources()
                    .getObject(GuiCommon.ITEM_VSZ_PEAK);
            row[1] = nbf.format(vszPeak / GuiCommon.BINARY_PREFIX) + " KB";
            model.addRow(row);
            row[0] = GuiCommon.getResources().getObject(
                    GuiCommon.ITEM_RSS_AVERAGE);
            row[1] = nbf.format(rssAve / count / GuiCommon.BINARY_PREFIX)
                    + " KB";
            model.addRow(row);
            row[0] = GuiCommon.getResources()
                    .getObject(GuiCommon.ITEM_RSS_PEAK);
            row[1] = nbf.format(rssPeak / GuiCommon.BINARY_PREFIX) + " KB";
            model.addRow(row);
        }

        /**
         * Check the size of values.
         *
         * @param current Comparison value
         * @param value Comparison value
         * @return Returns the larger value.
         */
        private float checkPeak(final float current, final float value) {
            if (current < value) {
                return value;
            }
            return current;
        }

        /**
         * Initialize the Java CPU chart.
         */
        private void initJavaCPU() {
            javaCpu = new DefaultCategoryDataset();
            JFreeChart chart = ChartFactory.createStackedAreaChart(GuiCommon
                    .getResources().getString(GuiCommon.GRAPH_JAVA_CPU), "",
                    "CPU(%)", javaCpu, PlotOrientation.VERTICAL, true, true,
                    true);
            chart.setBackgroundPaint(Color.WHITE);
            chart.getCategoryPlot().setBackgroundPaint(
                    new Color(GuiCommon.getBgColor()));
            javaCPUchart = new ChartPanel(chart);

            top.setLeftComponent(javaCPUchart);
        }

        /**
         * Draw the Java CPU chart.
         *
         * @param current The current value
         * @param last The last value
         * @param line current row number retrieved in a csv file
         */
        private void drawJavaCPU(final long[] current, final long[] last,
                final int line) {
            Runnable draw = new Runnable() {

                @Override
                public void run() {
                    SimpleDateFormat sdf = new SimpleDateFormat(
                            CHART_DATE_FORMAT);

                    double user = current[JAVA_CPU_USER] - last[JAVA_CPU_USER];
                    double system = current[JAVA_CPU_SYSTEM]
                            - last[JAVA_CPU_SYSTEM];

                    javaCpu.addValue((user / (user + system)) * 100, "user",
                            sdf.format(current[COLLECT_DATE]));
                    javaCpu.addValue((system / (user + system)) * 100,
                            "system", sdf.format(current[COLLECT_DATE]));

                    if (line > ARCHIVE_FILE) {
                        line(javaCPUchart.getChart().getCategoryPlot(),
                                sdf.format(current[COLLECT_DATE]));
                    }
                }
            };

            if (SwingUtilities.isEventDispatchThread()) {
                draw.run();
            } else {
                SwingUtilities.invokeLater(draw);
            }
        }

        /**
         * Initialize the System CPU chart.
         */
        private void initSystemCPU() {
            systemCpu = new DefaultCategoryDataset();
            JFreeChart chart = ChartFactory.createStackedAreaChart(GuiCommon
                    .getResources().getString(GuiCommon.GRAPH_SYSTEM_CPU), "",
                    "CPU(%)", systemCpu, PlotOrientation.VERTICAL, true, true,
                    true);
            chart.setBackgroundPaint(Color.WHITE);
            chart.getCategoryPlot().setBackgroundPaint(
                    new Color(GuiCommon.getBgColor()));
            sysCPUchart = new ChartPanel(chart);

            top.setRightComponent(sysCPUchart);
        }

        /**
         * Draw the System CPU chart.
         *
         * @param current The current value
         * @param last The last value
         * @param line current row number retrieved in a csv file
         */
        private void drawSystemCPU(final long[] current, final long[] last,
                final int line) {
            Runnable draw = new Runnable() {

                @Override
                public void run() {
                    SimpleDateFormat sdf = new SimpleDateFormat(
                            CHART_DATE_FORMAT);

                    float user = current[CPU_USER] - last[CPU_USER];
                    float nice = current[CPU_USER_NICE] - last[CPU_USER_NICE];
                    float system = current[CPU_SYSTEM] - last[CPU_SYSTEM];
                    float idle = current[CPU_IDLE] - last[CPU_IDLE];
                    float io = current[CPU_IO] - last[CPU_IO];
                    float irq = current[CPU_IRQ] - last[CPU_IRQ];
                    float soft = current[CPU_SOFT_IRQ] - last[CPU_SOFT_IRQ];
                    float steal = current[CPU_STEAL] - last[CPU_STEAL];
                    float guest = current[CPU_GUEST] - last[CPU_GUEST];

                    double denominator = user + nice + system + idle + io + irq
                            + soft + steal + guest;

                    systemCpu.addValue(user / denominator * 100, "user",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(nice / denominator * 100, "nice",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(system / denominator * 100, "system",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(idle / denominator * 100, "idle",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(io / denominator * 100, "I/O",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(irq / denominator * 100, "irq",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(soft / denominator * 100, "softirq",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(steal / denominator * 100, "steal",
                            sdf.format(current[COLLECT_DATE]));
                    systemCpu.addValue(guest / denominator * 100, "guest",
                            sdf.format(current[COLLECT_DATE]));

                    if (line > ARCHIVE_FILE) {
                        line(sysCPUchart.getChart().getCategoryPlot(),
                                sdf.format(current[COLLECT_DATE]));
                    }
                }
            };

            if (SwingUtilities.isEventDispatchThread()) {
                draw.run();
            } else {
                SwingUtilities.invokeLater(draw);
            }
        }

        /**
         * Initialize the java memory chart.
         */
        private void initJavaMemory() {
            javaMem = new DefaultCategoryDataset();
            JFreeChart chart = ChartFactory.createLineChart(GuiCommon
                    .getResources().getString(GuiCommon.GRAPH_JAVA_MEMORY), "",
                    "Memory(KB)", javaMem, PlotOrientation.VERTICAL, true,
                    true, true);
            chart.setBackgroundPaint(Color.WHITE);
            chart.getCategoryPlot().setBackgroundPaint(
                    new Color(GuiCommon.getBgColor()));
            memChart = new ChartPanel(chart);

            bottom.setLeftComponent(memChart);
        }

        /**
         * Draw the Java memory chart.
         *
         * @param current The current value
         * @param line current row number retrieved in a csv file
         */
        private void drawJavaMemory(final long[] current, final int line) {
            Runnable draw = new Runnable() {

                @Override
                public void run() {
                    SimpleDateFormat sdf = new SimpleDateFormat(
                            CHART_DATE_FORMAT);
                    javaMem.addValue(current[JAVA_RSS]
                            / GuiCommon.BINARY_PREFIX, "RSS(KB)",
                            sdf.format(current[COLLECT_DATE]));
                    javaMem.addValue(current[JAVA_VSZ]
                            / GuiCommon.BINARY_PREFIX, "VSZ(KB)",
                            sdf.format(current[COLLECT_DATE]));

                    if (line > ARCHIVE_FILE) {
                        line(memChart.getChart().getCategoryPlot(),
                                sdf.format(current[COLLECT_DATE]));
                    }
                }
            };

            if (SwingUtilities.isEventDispatchThread()) {
                draw.run();
            } else {
                SwingUtilities.invokeLater(draw);
            }
        }

        /**
         * Initialize the Threads chart.
         */
        private void initThreads() {
            this.threads = new DefaultCategoryDataset();
            this.safeTime = new DefaultCategoryDataset();
            JFreeChart chart = ChartFactory.createLineChart(GuiCommon
                    .getResources().getString(GuiCommon.GRAPH_THREADS), "", "",
                    this.threads, PlotOrientation.VERTICAL, true, true, true);
            chart.setBackgroundPaint(Color.WHITE);
            chart.getCategoryPlot().setBackgroundPaint(
                    new Color(GuiCommon.getBgColor()));

            CategoryPlot plot = chart.getCategoryPlot();
            plot.setDataset(2, this.safeTime);
            plot.mapDatasetToRangeAxis(2, 2);
            NumberAxis axis = new NumberAxis("Time(msec)");
            axis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
            plot.setRangeAxis(2, axis);
            plot.setDatasetRenderingOrder(DatasetRenderingOrder.REVERSE);

            LineAndShapeRenderer renderer = new LineAndShapeRenderer();
            renderer.setSeriesShapesVisible(0, false);
            renderer.setSeriesShapesVisible(1, false);
            plot.setRenderer(2, renderer);

            threadChart = new ChartPanel(chart);

            bottom.setRightComponent(threadChart);
        }

        /**
         * Draw the threads chart.
         *
         * @param current The current value
         * @param last The last value
         * @param line current row number retrieved in a csv file
         */
        private void drawThreads(final long[] current, final long[] last,
                final int line) {
            Runnable draw = new Runnable() {

                @Override
                public void run() {
                    SimpleDateFormat sdf = new SimpleDateFormat(
                            CHART_DATE_FORMAT);

                    threads.addValue(current[THREADS_LIVE], "threads",
                            sdf.format(current[COLLECT_DATE]));
                    threads.addValue(current[CONFLICT_MONITOR]
                            - last[CONFLICT_MONITOR], "monitor",
                            sdf.format(current[COLLECT_DATE]));
                    threads.addValue(current[SAFE_POINTS] - last[SAFE_POINTS],
                            "safepoints", sdf.format(current[COLLECT_DATE]));
                    // safepointTime
                    safeTime.addValue(current[SAFE_POINT_TIME]
                            - last[SAFE_POINT_TIME], "safepointtime(msec)",
                            sdf.format(current[COLLECT_DATE]));

                    if (line > ARCHIVE_FILE) {
                        line(threadChart.getChart().getCategoryPlot(),
                                sdf.format(current[COLLECT_DATE]));
                    }
                }
            };

            if (SwingUtilities.isEventDispatchThread()) {
                draw.run();
            } else {
                SwingUtilities.invokeLater(draw);
            }
        }

        /**
         * Draw a vertical line on the chart.
         *
         * @param plot To draw a line plot
         * @param date Logging date
         */
        private void line(final CategoryPlot plot, final String date) {
            CategoryMarker marker = new CategoryMarker(date);
            marker.setPaint(Color.WHITE);
            marker.setAlpha(ALPHA_TRANSPARENCY);
            plot.addDomainMarker(marker);
        }

        /**
         * Hide unwanted items out of the X axis label.
         *
         * @param ca CategoryAxis
         * @param hideLabel Hide item list
         */
        private void hideCategoryLabel(final CategoryAxis ca,
                final List<String> hideLabel) {
            ca.setAxisLineVisible(false);
            ca.setCategoryLabelPositions(CategoryLabelPositions
                    .createUpRotationLabelPositions(Math.PI / ANGLE));

            for (String label : hideLabel) {
                ca.setTickLabelPaint(label, new Color(0, 0, 0, 0));
            }
        }
    }
}
